MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN=1,NU=1,OMEGA=1
  DOUBLE PRECISION EPSTOL

CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)
    USE Q1DAMODULE
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,NU+OMEGA) :: Z
    DOUBLE PRECISION A,B,EPS,ANS,ERR,RPAR
    INTEGER KF,IFLG,IPAR
    DIMENSION RPAR(1),IPAR(1)
    B = T*T*Y(1)
    A = T*Y(1)
    EPS = EPSTOL
    CALL Q1DA(F,A,B,EPS,ANS,ERR,KF,IFLG,RPAR,IPAR)
	!PRINT *,'ERR= ',ERR
	!PRINT *,'IFLG= ',IFLG
    DY(1) = (ANS - 1D0) / (T*T*T)
    RETURN
  END SUBROUTINE DDES

  SUBROUTINE HISTORY(T,Y,DERIVATIVE)
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
	LOGICAL :: DERIVATIVE
    INTENT(IN)  :: T,DERIVATIVE
    INTENT(OUT) :: Y
	IF(.NOT. DERIVATIVE) THEN
      Y(1) = 1D0 / T
	ELSE
      Y(1) = -1D0 / (T*T)
	END IF
    RETURN
  END SUBROUTINE HISTORY

  SUBROUTINE BETA(T,Y,BVAL)
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
    DOUBLE PRECISION, DIMENSION(NU+OMEGA) :: BVAL
      BVAL(1) = T
      BVAL(2) = BVAL(1)
    RETURN
  END SUBROUTINE BETA

  SUBROUTINE CHECK(T,Y,ERROR,ABSERR,RELERR)
    DOUBLE PRECISION :: T,ABSERR,RELERR
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
    DOUBLE PRECISION :: EXACT,ERROR
    EXACT = 1D0/T
!   COMPUTE THE ERROR OVERRUN.
    ERROR = ABS((Y(1)-EXACT))!/(ABSERR+RELERR*ABS(EXACT)))
    WRITE(7,111) T,ERROR
    111 FORMAT(' T/Overrun = ',2D20.10)
    RETURN
  END SUBROUTINE CHECK

  SUBROUTINE F(S,RPAR,IPAR,FVAL)
!   INTEGRAND EVALUATION FOR Q1DA
    USE DDVERK90_M
    IMPLICIT NONE
    DOUBLE PRECISION S,RPAR,FVAL
    INTEGER IPAR,IQUEUE
    DIMENSION RPAR(1),IPAR(1)
    DOUBLE PRECISION, DIMENSION(NEQN) :: YOFT,DYOFT
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
    DOUBLE PRECISION, DIMENSION(NEQN) :: DY
!   EVALUATE THE INTEGRAND FOR Q1DA.
!   CHECK IF THE FIRST INTEGRATION STEP HAS BEEN COMPLETED.
!   IF NOT, USE THE HISTORY FUNCTION TO DEFINE Y AND Y'.
    CALL DDVERK_IQUEUE(IQUEUE)
    IF (IQUEUE == 0) THEN
       CALL HISTORY(S,Y,.FALSE.)
       CALL HISTORY(S,DY,.TRUE.)
       FVAL = S*S*S*Y(1)*DY(1)
       RETURN
    ENDIF
    IF (S <= 1D0) THEN
!      FVAL = S*S*S*(1/S)*(-1/S**2)
       FVAL = -1D0
    ELSE
!      USE THE SOLUTION QUEUE TO INTERPOLATE YOFT.
       CALL DDVERK_USER(S,YOFT,DYOFT)
       FVAL = S*S*S*YOFT(1)*DYOFT(1)
    ENDIF
    RETURN
    END SUBROUTINE F
END MODULE define_DDEs

!******************************************************************

PROGRAM Example_11 

! Solves the integral equation
! y'(t) = [int{s^3 y(s) y'(s) ds, s=t*y(t),...,t*y^2(t)} - 1] / t^3
! y(t)  = 1
! The DDE is defined in the module define_DDEs. The problem is
! solved here with DKLAG6 and its output written to a file.
! Using the auxilary function getplotdata, it is then easy to
! import the data into Matlab and plot it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   NU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so
  ! that they can be used for dimensioning arrays here.
  ! They are passed to the solver in the array NVAR.

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(2) :: NLAGS = (/NU,OMEGA/)

  TYPE(DDVERK_SOL) :: SOL ! Solution in Matlab style. 
  ! The fields of SOL are expressed in terms of the
  ! number of differential equations, NEQN, and the 
  ! number of output points, NPTS:

  !   SOL%NPTS         -- NPTS,number of output points.
  !
  !   SOL%T(NPTS)      -- values of independent variable, T.  
  !
  !   SOL%Y(NPTS,NEQN) -- values of dependent variable, Y,
  !                       corresponding to values of SOL%T. 
  TYPE(DDVERK_OPTS) :: OPTS
  TYPE(DDVERK_INT) :: YINT

  ! Local variables:
  INTEGER :: I,J
  DOUBLE PRECISION :: ERROR,MAXERROR,ABSERR,RELERR

  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT
  DOUBLE PRECISION :: T0=1.0D0,TFINAL=2.0D0
  ! Local variable:
  !INTEGER :: I

 
  ABSERR = 1D-14
  RELERR = 1D-8
  !EPSTOL = MIN(ABSERR,RELERR) / 100D0
  EPSTOL = RELERR !MAX(ABSERR,RELERR) / 100D0


  !ABSERR = 1D-6
  !RELERR = 1D-3
  !EPSTOL = MIN(ABSERR,RELERR) / 100D0
  !EPSTOL = 1D-2


  OPTS = DDVERK_SET(RE=RELERR,AE=ABSERR,NEUTRAL=.TRUE.,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(TOL=ABSERR,NEUTRAL=.TRUE.)
  !OPTS = DDVERK_SET(ABS_TOL=ABSERR,NEUTRAL=.TRUE.)
  SOL  = DDVERK90(NVAR,NLAGS,DDES,BETA,HISTORY,&
                   (/ T0, TFINAL /),OPTIONS=OPTS)

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN
 
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
     YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_11.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(2D12.4)') TINT(I),YINT%YT(I,1)
     END DO


     PRINT *,' Normal return from DDVERK_SOLVER with results'
     PRINT *," written to the file 'Example_11.dat'."
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' from the folder holding this file by'
     PRINT *,' '
     PRINT *," >> [t,y] = getplotdata('Example_11.dat');"
     PRINT *,' '
     PRINT *,' and then plotted with, e.g., '
     PRINT *,' '
     PRINT *,' >> plot(t,y)'
     PRINT *,' '

     MAXERROR = 0D0
     DO I = 1,SOL%NPTS
        CALL CHECK(SOL%T(I),SOL%Y(I,:),ERROR,ABSERR,RELERR)
        MAXERROR = MAX(ERROR,MAXERROR)
     ENDDO
     PRINT *,' '
     PRINT *,' Comparison to the analytical solution shows that'
     PRINT *,' at all steps the error overrun is less than ', &
     MAXERROR
     PRINT *,' '
     CALL DDVERK_PRINT_STATS(SOL)
  ELSE

     PRINT *,' Abnormal return from DDVERK_SOLVER with FLAG = ',&
     SOL%FLAG

  ENDIF

  STOP
END PROGRAM Example_11
