MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN= 6,MU=2,NEF=2
  DOUBLE PRECISION,PARAMETER :: Alpha=1.8D0,Beta=20.D0 , &
			Gamma=0.002D0,r=5.D4,s=1.D5,P_t0=35.D0,t1=197.D0

  DOUBLE PRECISION :: Htt0,Htt1
CONTAINS


FUNCTION f1(A,B,C) RESULT(D)
	DOUBLE PRECISION :: A,B,C,D
	D=A*B+C
END FUNCTION f1

FUNCTION f2(A,B) RESULT(D)
	DOUBLE PRECISION :: A,B,D
	D=1.D-12+A+B
END FUNCTION f2

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,MU) :: Z
	INTENT(IN) :: T,Y,Z
	INTENT(OUT) :: DY
	!DOUBLE PRECISION :: A,B,C,f1,f2
	!f1(A,B,C)=A*B+C
	!f2(A,B) = 1.D-12+A+B

    DY(1) = -r*Y(1)*Y(2)-s*Y(1)*Y(4)
	DY(2) = -r*Y(1)*Y(2)+Alpha*r*Z(1,1)*Z(2,1)*Htt0
	DY(3) = r*Y(1)*Y(2)
	DY(4) = -s*Y(1)*Y(4)-Gamma*Y(4)+Beta*r*Z(1,2)*Z(2,2)*Htt1
	DY(5) = Htt0*f1(Y(1),Y(2),Y(3))/f1(Z(1,1),Z(2,1),Z(3,1))
	DY(6) = Htt1*f2(Y(2),Y(3))/f2(Z(2,2),Z(3,2))
 
    RETURN
  END SUBROUTINE DDES

  SUBROUTINE EF(T,Y,DY,Z,G)

     DOUBLE PRECISION :: T
     DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
     DOUBLE PRECISION, DIMENSION(NEQN,MU) :: Z
     DOUBLE PRECISION, DIMENSION(NEF) :: G
	 INTENT(IN) :: T,Y,DY,Z
	 INTENT(OUT) :: G

      G(1) = T-P_t0
	  G(2) = T-t1

     RETURN
  END SUBROUTINE EF

  SUBROUTINE CHNG(EVENTS,TEVENT,YEVENT,DYEVENT,HINIT, &
                  DIRECTION,ISTERMINAL,QUIT,IS_CHANGED)
  ! Function to change a flag so that the DDE model will
  ! be evaluated in subroutine DDES instead of the ODE model.

	 LOGICAL,DIMENSION(NEF) :: EVENTS
     INTEGER, DIMENSION(NEF) :: DIRECTION
     DOUBLE PRECISION :: TEVENT,HINIT
     DOUBLE PRECISION, DIMENSION(NEQN) :: YEVENT,DYEVENT
     LOGICAL :: QUIT,IS_CHANGED
     LOGICAL, DIMENSION(NEF) :: ISTERMINAL
	 INTENT(IN) :: EVENTS,TEVENT
	 INTENT(INOUT) :: YEVENT,DYEVENT,HINIT,DIRECTION,&
	                  ISTERMINAL,QUIT,IS_CHANGED

	 IF (EVENTS(1)) THEN
		Htt0 = 1.D0
	 ELSE
		Htt1 = 1.D0
	 END IF
     IS_CHANGED = .TRUE.

    RETURN
  END SUBROUTINE CHNG

  SUBROUTINE LAG_BETA(T,Y,BVAL)
    
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
    DOUBLE PRECISION, DIMENSION(MU) :: BVAL
	INTENT(IN)  :: T,Y
	INTENT(OUT) :: BVAL

      BVAL(1) = Y(5)
	  BVAL(2) = Y(6)

    RETURN
  END SUBROUTINE LAG_BETA

END MODULE define_DDEs

!******************************************************************

PROGRAM Example_14

! 
!
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_14.M imports the data into
! Matlab and plots it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   MU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the arrays NVAR and NLAGS

  INTEGER, DIMENSION(2) :: NVAR = (/NEQN,NEF/)
  INTEGER, DIMENSION(1) :: NLAGS = (/MU/)

  TYPE(DDVERK_SOL) :: SOL 
  TYPE(DDVERK_INT) :: YINT
  TYPE(DDVERK_OPTS) :: OPTS

  DOUBLE PRECISION :: T0= 0.D0,TFINAL=300.D0
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT

  ! Local variables:
  INTEGER :: I,J

  DOUBLE PRECISION, DIMENSION(NEQN) :: HISTORY= (/5.D-6,1.D-15,0.D0,0.D0,0.D0,0.D0/)

  Htt0=0.D0
  Htt1=0.D0

!  OPTS = DDVERK_SET(RE=1D-7,AE_VECTOR=(/1D-14,1D-14,1D-14,1D-14,1D-7,1D-7/), &
!						INTERPOLATION=.TRUE.,DIRECTION=(/ +1,+1 /))
  OPTS = DDVERK_SET(RE=1D-4,AE_VECTOR=(/1D-14,1D-14,1D-14,1D-14,1D-7,1D-7/), &
						INTERPOLATION=.TRUE.,DIRECTION=(/ +1,+1 /))

!OPTS = DDVERK_SET(RE=1D-7,AE=1.D-10, &
!						INTERPOLATION=.TRUE.,DIRECTION=(/ +1,+1 /))

   ! OPTS = DDVERK_SET(TOL=1D-12, &
!						INTERPOLATION=.TRUE.,DIRECTION=(/ +1,+1 /))
  !OPTS = DDVERK_SET(ABS_TOL=1D-18,REL_WEIGHTS=(/1.D0,1.D0,1.D0,1.D0,1.D20,1.D13/), &
!						INTERPOLATION=.TRUE.,DIRECTION=(/ +1,+1 /))

  SOL = DDVERK90(NVAR,NLAGS,DDES,LAG_BETA,HISTORY, &
         (/ T0,TFINAL /),OPTIONS=OPTS,EVENT_FCN=EF,CHANGE_FCN=CHNG) 

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN

     ! Form values for phase plane plot:
     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
	 YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
		
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_14.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(7D12.4)') TINT(I),(YINT%YT(I,J),J=1,NEQN)
     END DO

  	 PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file 'Example_14.dat'"
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' and plotted by'
     PRINT *,' '
     PRINT *," >> [t,y] = Example_14;"
     PRINT *,' '

	 CALL DDVERK_PRINT_STATS(SOL)

	 PRINT *,'at T=',SOL%T(SOL%NPTS)
	 PRINT *,' Y1=',SOL%Y(SOL%NPTS,1),' Y2=',SOL%Y(SOL%NPTS,2) 
	 PRINT *,' Y3=',SOL%Y(SOL%NPTS,3) ,' Y4=',SOL%Y(SOL%NPTS,4) 
	 PRINT *,' Y5=',SOL%Y(SOL%NPTS,5) ,' Y6=',SOL%Y(SOL%NPTS,6) 
  ELSE
 
     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG
 
  ENDIF
    
  STOP
END PROGRAM Example_14
