MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN= 10,MU=5
  DOUBLE PRECISION,DIMENSION(39),PARAMETER:: a=(/83D0,5.D0,6.6D14, &
													3.D11,0.4D0,2.5D7, &
													0.5D-12,2.3D9,0.052D0, &
													0.15D0,9.4D9,1.D-15, &
													1.2D0,2.7D16,2.D0, &
													5.3D27,1.D0,1.D-18, &
													2.7D16,2.D0,8.D28, &
													1.D0,1.D-19,5.3D33, &
													16.D0,1.6D14,0.4D0, &
													1D-18,8.D32,16.D0, &
													0.1D0,1.D-18,1.7D30, &
													3.D0,0.4D0,4.3D-22, &
													0.85D7,8.6D11,0.043D0/)
  DOUBLE PRECISION,DIMENSION(5),PARAMETER :: TAU=(/0.6D0,0.6D0,2.D0, &
											   2.D0,3.D0/)

CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,MU) :: Z
	INTENT(IN) :: T,Y,Z
	INTENT(OUT) :: DY
	DOUBLE PRECISION :: xi

    DY(1) = a(1)*Y(2)+a(2)*a(3)*Y(2)*Y(7)-a(4)*Y(1)*Y(10)-a(5)*Y(1)- &
		    a(6)*Y(1)*(a(7)-Y(2)-Y(3))
	DY(2) = a(8)*Y(1)*(a(7)-Y(2)-Y(3))-a(3)*Y(2)*Y(7)-a(9)*Y(2)
	DY(3) = a(3)*Y(2)*Y(7)+a(9)*Y(2)-a(10)*Y(3)
	xi = 1-Y(3)/a(7)
	DY(4) = a(11)*a(12)*Y(1)-a(13)*Y(4)
	DY(5) = a(14)*(xi*a(15)*Z(4,1)*Z(5,1)-Y(4)*Y(5)) - &
			a(16)*Y(4)*Y(5)*Y(7)+a(17)*(a(18)-Y(5))
	DY(6) = a(19)*(xi*a(20)*Z(4,2)*Z(6,2)-Y(4)*Y(6)) - &
			a(21)*Y(4)*Y(6)*Y(8)+a(22)*(a(23)-Y(6))
	DY(7) = a(24)*(xi*a(25)*Z(4,3)*Z(5,3)*Z(7,3)-Y(4)*Y(5)*Y(7)) - &
			a(26)*Y(2)*Y(7)+a(27)*(a(28)-Y(7))
	DY(8) = a(29)*(xi*a(30)*Z(4,4)*Z(6,4)*Z(8,4)-Y(4)*Y(6)*Y(8)) + &
			a(31)*(a(32)-Y(8))
	DY(9) = a(33)*xi*a(34)*Z(4,5)*Z(6,5)*Z(8,5)+a(35)*(a(36)-Y(9))
	DY(10)= a(37)*Y(9)-a(38)*Y(10)*Y(1)-a(39)*Y(10)

    RETURN
  END SUBROUTINE DDES

END MODULE define_DDEs

!******************************************************************

PROGRAM Example_15

! 
!
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function Example_15.M imports the data into
! Matlab and plots it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   MU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the arrays NVAR and NLAGS

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(1) :: NLAGS = (/MU/)

  TYPE(DDVERK_SOL) :: SOL 
  TYPE(DDVERK_INT) :: YINT
  TYPE(DDVERK_OPTS) :: OPTS

  DOUBLE PRECISION :: T0= 0.0D0,TFINAL=114.5D0 !130.D0
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT

  ! Local variables:
  INTEGER :: I,J

  DOUBLE PRECISION, DIMENSION(MU) :: LAGS= TAU
  DOUBLE PRECISION, DIMENSION(NEQN) :: HISTORY= (/ 2.9D-16 ,0.D0,0.D0, &
												0.D0,a(18),a(23), &
												a(28),a(32),a(36), &
												a(37)*a(36)/a(39)/)

  !OPTS = DDVERK_SET(RE=1D-11,AE=1D-31,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(RE=1D-8,AE=1D-13,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(RE=1D-11,AE_VECTOR=(/1.D-10,1.D-12,1D-12,&
!										1.D-15,1.D-14,1D-15, &
!										1.D-8,1.D-11,1D-14, &
!										1.D-7/),INTERPOLATION=.TRUE.)

  OPTS = DDVERK_SET(RE=1D-11,AE=1D-31,INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(RE=1D-11,AE=1D-27,INTERPOLATION=.TRUE.)

  !OPTS = DDVERK_SET(ABS_TOL=1D-23,REL_WEIGHTS=(/1.D5,1.D3,1D3,&
!										1.D2,1.D2,1D1, &
!										1.D8,1.D5,1D0, &
!										1.D9/),INTERPOLATION=.TRUE.)
!  OPTS = DDVERK_SET(ABS_TOL=1D-24,REL_WEIGHTS=(/1.D0,1.D3,1D0,&
!										1.D0,1.D4,1D4, &
!										1.D8,1.D5,1D5, &
!										1.D9/),INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(ABS_TOL=1D-8,REL_WEIGHTS=(/1.D-32,1.D-30,1D-25,&
!										1.D-27,1.D-19,1D-20, &
!										1.D-19,1.D-18,1D-22, &
!										1.D-16/),INTERPOLATION=.TRUE.)
  !OPTS = DDVERK_SET(RE=1D-11,AE=1D-19,INTERPOLATION=.TRUE.)

  SOL = DDVERK90(NVAR,NLAGS,DDES,LAGS,HISTORY, &
         (/ T0,TFINAL /),OPTIONS=OPTS) 

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN

     ! Form values for phase plane plot:
     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
	 YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
		
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='Example_15.dat')
     DO I = 1,NOUT
       WRITE(UNIT=6,FMT='(11D12.4)') TINT(I),(YINT%YT(I,J),J=1,NEQN)
     END DO

  	 PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file 'Example_15.dat'"
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' and plotted by'
     PRINT *,' '
     PRINT *," >> [t,y] = Example_15;"
     PRINT *,' '

	 !PRINT *,'T=',SOL%T(SOL%NPTS),' Y1=',SOL%Y(2,1),' Y2=',SOL%Y(2,2)
	 PRINT *,'T=',SOL%T(SOL%NPTS),' Y1=',SOL%Y(SOL%NPTS,1),' Y3=',SOL%Y(SOL%NPTS,3)

	 CALL DDVERK_PRINT_STATS(SOL)

  ELSE
 
     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG
 
  ENDIF
    
  STOP
END PROGRAM Example_15
