MODULE define_DDEs

  IMPLICIT NONE
  
  INTEGER, PARAMETER :: NEQN= !3,MU=2


CONTAINS

  SUBROUTINE DDES(T,Y,Z,DY)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y,DY
    DOUBLE PRECISION, DIMENSION(NEQN,MU) :: Z
	INTENT(IN) :: T,Y,Z
	INTENT(OUT) :: DY

    DY(1) = !-Y(1)*Z(2, 1) +Z(2, 2)
    !DY(2) = Y(1)* Z(2,1) - Y(2)
    !DY(3) = Y(2) - Z(2, 2)
	   

    RETURN
  END SUBROUTINE DDES

  SUBROUTINE HISTORY(T,Y,DERIVATIVE)

    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
	LOGICAL :: DERIVATIVE
	INTENT(IN)  :: T,DERIVATIVE
	INTENT(OUT) :: Y
	IF(.NOT. DERIVATIVE) THEN
      Y(1) = !SIN(T)
	ELSE
      Y(1) = !COS(T)
	END IF

    RETURN
  END SUBROUTINE HISTORY

  SUBROUTINE BETA(T,Y,BVAL)
    
    DOUBLE PRECISION :: T
    DOUBLE PRECISION, DIMENSION(NEQN) :: Y
    DOUBLE PRECISION, DIMENSION(MU+OMEGA) :: BVAL
	INTENT(IN)  :: T,Y
	INTENT(OUT) :: BVAL

      BVAL(1) = !T*Y(1)**2

    RETURN
  END SUBROUTINE BETA

END MODULE define_DDEs

!******************************************************************

PROGRAM pattern_prg

! 
!
! The DDE is defined in the module define_DDEs.  The problem
! is solved here with DDVERK90 and its output written to a
! file.  The auxilary function pattern_prg.M imports the data into
! Matlab and plots it.

  USE define_DDEs
  USE DDVERK90_M

  IMPLICIT NONE

  ! The quantities
  !
  !   NEQN  = number of equations
  !   MU = number of delays
  !
  ! are defined in the module define_DDEs as PARAMETERs so 
  ! they can be used for dimensioning arrays here. They are 
  ! passed to the solver in the arrays NVAR and NLAGS

  INTEGER, DIMENSION(1) :: NVAR = (/NEQN/)
  INTEGER, DIMENSION(1) :: NLAGS = (/MU/)

  TYPE(DDVERK_SOL) :: SOL 
  TYPE(DDVERK_INT) :: YINT
  TYPE(DDVERK_OPTS) :: OPTS

  DOUBLE PRECISION :: T0= !0.0D0,TFINAL=55.0D0
  ! Prepare output points
  INTEGER, PARAMETER :: NOUT=1000
  DOUBLE PRECISION, DIMENSION(NOUT) :: TINT

  ! Local variables:
  INTEGER :: I

  DOUBLE PRECISION, DIMENSION(MU) :: LAGS= !(/ 1.0D0 ,10.0D0/)
  DOUBLE PRECISION, DIMENSION(NEQN) :: HISTORY= !(/ 5.0D0 ,0.1D0,1.0D0/)

  OPTS = DDVERK_SET(RE=1D-6,AE=1D-6,INTERPOLATION=.TRUE.)

  SOL = DDVERK90(NVAR,NLAGS,DDES,LAGS,HISTORY, &
         (/ T0,TFINAL /),OPTIONS=OPTS) 

  ! Was the solver successful?
  IF (SOL%FLAG == 0) THEN

     ! Form values for phase plane plot:
     TINT = (/ (T0+(I-1)*((TFINAL-T0)/(NOUT-1)), I=1,NOUT) /)
	 YINT = DDVERK_VAL(TINT,SOL,DERIVATIVES=.FALSE.)
		
     ! Write the solution to a file for subsequent plotting
     ! in Matlab.
     OPEN(UNIT=6, FILE='pattern_prg.dat')
     DO I = 1,NOUT
       !WRITE(UNIT=6,FMT='(4D12.4)') TINT(I),YINT%YT(I,1),&
	    !                            YINT%YT(I,2),YINT%YT(I,3)
     END DO

  	 PRINT *,' Normal return from DDVERK90 with results'
     PRINT *," written to the file 'pattern_prg.dat'"
     PRINT *,' '
     PRINT *,' These results can be accessed in Matlab'
     PRINT *,' and plotted by'
     PRINT *,' '
     PRINT *," >> [t,y] = pattern_prg;"
     PRINT *,' '

	 CALL DDVERK_PRINT_STATS(SOL)

  ELSE
 
     PRINT *,' Abnormal return from DDVERK90 with FLAG = ',&
     SOL%FLAG
 
  ENDIF
    
  STOP
END PROGRAM pattern_prg
